# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#          Roberto Alsina <roberto.alsina@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""The user interface for the Ubuntu One Installer."""

import subprocess

import gettext

from PyQt4 import QtGui, QtCore

from ubuntu_sso.qt.gui import (
    SSOWizardPage,
    EmailVerificationPage,
    ErrorPage,
    ForgottenPasswordPage,
    ResetPasswordPage,
)
from ubuntu_sso.qt.current_user_sign_in_ui import \
    Ui_CurrentUserSignInPage
from ubuntu_sso.qt.email_verification_ui import \
    Ui_EmailVerificationPage
from ubuntu_sso.qt.error_message_ui import Ui_ErrorPage
from ubuntu_sso.qt.terms_and_conditions_ui import Ui_TosPage
from ubuntu_sso.qt.success_message_ui import Ui_SuccessPage
from ubuntu_sso.qt.forgotten_password_ui import \
    Ui_ForgottenPasswordPage
from ubuntu_sso.qt.reset_password_ui import Ui_ResetPasswordPage
from ubuntu_sso.qt.controllers import (
    ChooseSignInController,
    EmailVerificationController,
    ErrorController,
    ResetPasswordController,
    SetUpAccountController,
    SuccessController,
    TosController,
)

from ubuntuone.controlpanel.gui.qt.loadingoverlay import LoadingOverlay

from ubuntuone.platform.credentials import (
    APP_NAME,
    CredentialsManagementTool,
    TC_URL,
)

from ubuntuone_installer.logger import setup_logging
from ubuntuone_installer.gui.qt.ui import (
    choose_sign_in_ui,
    congratulations_ui,
    license_ui,
    setup_account_ui,
)

# Module used to include the resources into this file
# pylint: disable=W0611
from ubuntuone_installer.gui.qt.ui import images_rc
# pylint: enable=W0611

from ubuntuone_installer.gui.qt.are_you_sure import AreYouSure
from ubuntuone_installer.gui.qt.currentuser import CurrentUserController
from ubuntuone_installer.gui.qt.forgotten import ForgottenPasswordController
from ubuntuone_installer.gui.qt.local_folders import LocalFoldersPage
from ubuntuone_installer.gui.qt.sync_now_or_later import SyncNowOrLaterPage
from ubuntuone_installer.gui.qt.setup_account import SetupAccountPage
from ubuntuone_installer.gui.qt.side_widget import SideWidget
from ubuntuone_installer.gui.qt.tos import TosPage

_ = gettext.gettext
SIGN_IN = _("Sign in to Ubuntu One")
SIGN_IN_SUBTITLE = _("Sign in with your existing Ubuntu One"
    " username and password.")
TITLE_STYLE = "<span style=\"font-size:24px\">%s</span>"

# Invalid name logger
# pylint: disable=C0103
logger = setup_logging('qt.gui')
# pylint: enable=C0103


class LicensePage(SSOWizardPage):
    """Wizard Page that displays the license info and links to the GPL."""

    def __init__(self, parent=None):
        super(LicensePage, self).__init__(license_ui.Ui_Form(), None, parent)
        self.header.setVisible(False)

    # Invalid names of Qt-inherited methods
    # pylint: disable=C0103

    def initializePage(self):
        """Setup UI details."""
        # Set the right texts and connections for buttons
        self.setButtonText(QtGui.QWizard.NextButton, _("Agree && Install"))
        self.setButtonText(QtGui.QWizard.CancelButton,
            _("Disagree && Cancel"))
        self.setButtonText(QtGui.QWizard.CustomButton1, _("&Print"))

        # This is just to catch an exception thrown when nothing
        # is connected to the signal. It's not an exceptional
        # condition at all.
        try:
            self.wizard().customButtonClicked.disconnect()
        except TypeError:
            pass

        self.wizard().customButtonClicked.connect(self.print_document)

        self.wizard().setButtonLayout([
            QtGui.QWizard.CancelButton,
            QtGui.QWizard.BackButton,
            QtGui.QWizard.Stretch,
            QtGui.QWizard.CustomButton1,
            QtGui.QWizard.NextButton,
            QtGui.QWizard.FinishButton])

    def print_document(self, button_id):
        """Print the document displayed in textBrowser."""
        if button_id == QtGui.QWizard.CustomButton1:
            document = self.ui.textBrowser.document()
            printer = QtGui.QPrinter(QtGui.QPrinter.HighResolution)

            dialog = QtGui.QPrintDialog(printer, self)

            dialog.setWindowTitle(_("Send to printer"))
            if dialog.exec_() != QtGui.QDialog.Accepted:
                return
            printer.setFullPage(True)
            printer.setPageSize(QtGui.QPrinter.A4)
            document.print_(printer)

    def nextId(self):
        """Return next page's ID."""
        if self._next_id is None:
            return super(LicensePage, self).nextId()
        return self._next_id


class SignInPage(SSOWizardPage):
    """Wizard Page that lets the user Sign into Ubuntu One."""

    # Invalid names of Qt-inherited methods
    # pylint: disable=C0103

    def initializePage(self):
        """Setup UI details."""
        self.ui.cancel_button.setText(_("Close window and setup later"))
        self.ui.cancel_button.clicked.connect(self.wizard().reject)
        # Layout without custom button 1,
        # without finish button
        # without cancel
        self.wizard().setButtonLayout([
            QtGui.QWizard.Stretch,
            QtGui.QWizard.NextButton])

        # Set existing_account_button as default when the page is shown.
        self.ui.existing_account_button.setDefault(True)
        self.ui.existing_account_button.style().unpolish(
            self.ui.existing_account_button)
        self.ui.existing_account_button.style().polish(
            self.ui.existing_account_button)

    def nextId(self):
        """Provide the next id."""
        return self.next


class CurrentUserSignInPage(SSOWizardPage):
    """Wizard Page that lets a current user Sign into Ubuntu One."""

    # Invalid names of Qt-inherited methods
    # pylint: disable=C0103

    def nextId(self):
        """Provide the next id."""
        return self.next

    def initializePage(self):
        """Setup UI details."""
        self.setButtonText(QtGui.QWizard.CancelButton,
            _("Cancel"))
        # Layout without custom button 1,
        # without finish button
        self.wizard().setButtonLayout([
            QtGui.QWizard.BackButton,
            QtGui.QWizard.Stretch])

        # Set sign_in_button as default when the page is shown.
        self.ui.sign_in_button.setDefault(True)
        self.ui.sign_in_button.style().unpolish(
            self.ui.sign_in_button)
        self.ui.sign_in_button.style().polish(
            self.ui.sign_in_button)


class SuccessPage(SSOWizardPage):
    """Shown after SSO login, before setup."""

    # Invalid names of Qt-inherited methods
    # pylint: disable=C0103

    def initializePage(self):
        """Setup UI details."""
        self.wizard().setButtonLayout([
            QtGui.QWizard.Stretch,
            QtGui.QWizard.NextButton,
            QtGui.QWizard.FinishButton])
        self.wizard().setButtonText(QtGui.QWizard.FinishButton,
            _("Skip tour, go to Dashboard"))

    def nextId(self):
        """Provide the next id."""
        return self.next


class CongratulationsPage(SSOWizardPage):
    """Final page of the wizard."""

    def __init__(self, parent=None):
        super(CongratulationsPage, self).__init__(
            congratulations_ui.Ui_Form(), None, parent)
        self.setFinalPage(True)

    # Invalid names of Qt-inherited methods
    # pylint: disable=C0103

    def initializePage(self):
        """Setup UI details."""
        # We need custom buttons
        self.wizard().setButtonText(QtGui.QWizard.FinishButton,
            _("Go to my Ubuntu One dashboard"))
        # Layout without custom button 1,
        self.wizard().setButtonLayout([
            QtGui.QWizard.Stretch,
            QtGui.QWizard.FinishButton])
        if self.field("sync_now").toBool() \
            or self.field("sync_custom").toBool():
            self.ui.progressContainer.setVisible(True)
        else:
            self.ui.progressContainer.setVisible(False)

        # Set FinishButton as default when the page is shown.
        self.wizard().button(QtGui.QWizard.FinishButton).setDefault(True)
        self.wizard().button(QtGui.QWizard.FinishButton).style().unpolish(
            self.wizard().button(QtGui.QWizard.FinishButton))
        self.wizard().button(QtGui.QWizard.FinishButton).style().polish(
            self.wizard().button(QtGui.QWizard.FinishButton))


class MainWindow(QtGui.QWizard):

    """The Main Window of the Installer wizard."""

    # Invalid constant names and Qt-inherited methods
    # pylint: disable=C0103

    LICENSE_PAGE = 0
    SIGNIN_PAGE = 1
    SYNC_NOW_OR_LATER_PAGE = 2
    CONGRATULATIONS_PAGE = 3

    # definition of the signals raised by the widget (for SSO)
    recoverableError = QtCore.pyqtSignal('QString', 'QString')
    loginSuccess = QtCore.pyqtSignal('QString', 'QString')
    registrationSuccess = QtCore.pyqtSignal('QString', 'QString')
    userCancellation = QtCore.pyqtSignal('QString')

    def __init__(self, close_callback=None):
        """Initialize this instance."""
        # Used to decide the next page dynamically
        self._next_id = None

        # Data for SSO
        self.app_name = APP_NAME
        self.tc_url = TC_URL
        # help_text is not displayed in the UI from design
        self.help_text = ""

        super(MainWindow, self).__init__()

        self.setWindowTitle(APP_NAME)
        self.setMinimumHeight(630)
        self.setMinimumWidth(800)
        self.setWizardStyle(self.ModernStyle)
        self.close_callback = close_callback

        self.setSideWidget(SideWidget())
        self.overlay = LoadingOverlay(self)
        self.overlay.hide()

        self.setOption(self.NoBackButtonOnStartPage, True)

        # PyQt doesn't support the (int, page) version of addPage, so
        # Add the pages in the right order
        # pylint: disable=C0103

        self.LICENSE_PAGE = LicensePage()
        self.LICENSE_PAGE_ID = self.addPage(self.LICENSE_PAGE)

        #SSO Pages
        title_page = TITLE_STYLE % SIGN_IN
        self.sign_in_controller = ChooseSignInController(
            title=title_page)
        self.sign_in_page = SignInPage(
            ui=choose_sign_in_ui.Ui_ChooseSignInPage(),
            controller=self.sign_in_controller,
            parent=self)

        self.setup_controller = SetUpAccountController(message_box=self)
        self.setup_account = SetupAccountPage(
            setup_account_ui.Ui_SetUpAccountPage(),
            self.setup_controller,
            parent=self)
        self.setup_account.ui.set_up_button.clicked.connect(
            self.overlay.show)

        title_page = TITLE_STYLE % _("Ubuntu One Terms of Service")
        self.tos = TosPage(Ui_TosPage(),
            TosController(tos_url=TC_URL,
                title=title_page),
            parent=self)

        self.email_verification = EmailVerificationPage(
                                                Ui_EmailVerificationPage(),
                                                EmailVerificationController())
        self.email_verification.next_button.clicked.connect(
            self.overlay.show)

        title_page = TITLE_STYLE % SIGN_IN
        self.current_user_controller = CurrentUserController(
            title=title_page, subtitle=SIGN_IN_SUBTITLE,
            message_box=self)
        self.current_user = CurrentUserSignInPage(Ui_CurrentUserSignInPage(),
                                                  self.current_user_controller,
                                                  parent=self)
        self.current_user.ui.sign_in_button.clicked.connect(
            self.overlay.show)

        self.success_controller = SuccessController()
        self.success = SuccessPage(Ui_SuccessPage(), self.success_controller,
                                   parent=self)
        self.error_controller = ErrorController()
        self.error = ErrorPage(Ui_ErrorPage(), self.error_controller)

        self.forgotten_pwd_controller = ForgottenPasswordController()
        self.forgotten = ForgottenPasswordPage(Ui_ForgottenPasswordPage(),
                                               self.forgotten_pwd_controller,
                                               parent=self)
        self.forgotten.ui.send_button.clicked.connect(
            self.overlay.show)

        self.reset_password_controller = ResetPasswordController()
        self.reset_password = ResetPasswordPage(Ui_ResetPasswordPage(),
                                                self.reset_password_controller,
                                                parent=self)
        self.reset_password.ui.reset_password_button.clicked.connect(
            self.overlay.show)

        self.SIGNIN_PAGE = self.addPage(self.sign_in_page)
        self.setup_account_page_id = self.addPage(self.setup_account)
        self.tos_page_id = self.addPage(self.tos)
        self.email_verification_page_id = self.addPage(self.email_verification)
        self.current_user_page_id = self.addPage(self.current_user)
        self.SUCCESS_PAGE = self.addPage(self.success)
        self.ERROR_PAGE = self.addPage(self.error)
        self.forgotten_password_page_id = self.addPage(self.forgotten)
        self.reset_password_page_id = self.addPage(self.reset_password)

        self.loginSuccess.connect(self.login_success_slot)
        self.registrationSuccess.connect(self.registration_success_slot)

        # End of SSO pages

        self.local_folders_page = LocalFoldersPage()
        self.local_folders_page_id = self.addPage(self.local_folders_page)
        self.SYNC_NOW_OR_LATER_PAGE = self.addPage(SyncNowOrLaterPage())
        self.CONGRATULATIONS_PAGE = self.addPage(CongratulationsPage())

        self.currentIdChanged.connect(self.on_currentIdChanged)
        # Set Wizard buttons style
        self.button(QtGui.QWizard.NextButton).setDefault(True)

        # Set Application Style Sheet
        app = QtGui.QApplication.instance()
        qss_file = QtCore.QFile(":/ubuntuone.qss")
        qss_file.open(QtCore.QFile.ReadOnly)
        stylesheet = QtCore.QLatin1String(qss_file.readAll())
        app.setStyleSheet(stylesheet)
        # Apply font to the entire application
        QtGui.QFontDatabase.addApplicationFont(':/Ubuntu-R.ttf')
        QtGui.QFontDatabase.addApplicationFont(':/Ubuntu-B.ttf')
        credtool = CredentialsManagementTool()
        d = credtool.find_credentials()
        d.addCallback(self.creds_cb)
        d.addErrback(lambda exc: logger.error(
            'Error while getting the credentials: %r', exc))

    # Invalid name "closeEvent"
    # pylint: disable=C0103

    def creds_cb(self, token):
        """Change wizard flow if we have credentials."""
        if not token:
            self.LICENSE_PAGE._next_id = self.SIGNIN_PAGE
        else:
            self.LICENSE_PAGE._next_id = self.local_folders_page_id

    def critical(self, *args, **kwargs):
        """Wrapper for QMessageBox.critical that hides the overlay."""
        self.overlay.hide()
        QtGui.QMessageBox.critical(*args, **kwargs)

    def next(self):
        """Show the next page to display."""
        self.overlay.hide()
        super(MainWindow, self).next()

    def nextId(self):
        """Selects which page to display next."""
        if self._next_id is not None:
            return self._next_id
        else:
            return QtGui.QWizard.nextId(self)

    def login_success_slot(self):
        """Called on successful login."""
        self._next_id = self.SUCCESS_PAGE
        self.next()
        self._next_id = self.local_folders_page_id

    def registration_success_slot(self):
        """Called on successful registration."""
        self._next_id = self.SUCCESS_PAGE
        self.next()
        self._next_id = self.local_folders_page_id

    def done(self, result):
        """The main window is being closed, call any custom callback."""
        # Finished, not cancelled
        if result == 1:  # Cancelled
            subprocess.Popen(["ubuntuone-control-panel-qt", ])
        elif self.currentId() != self.CONGRATULATIONS_PAGE:
            if AreYouSure(self).exec_():
                return
        if self.close_callback is not None:
            self.close_callback()
        QtGui.QWizard.done(self, result)
        self.userCancellation.emit(self.app_name)

    def on_currentIdChanged(self, page_id):
        """The current page changed."""
        if page_id < self.SIGNIN_PAGE:
            self.sideWidget().ui.states_frame.hide()
            self.sideWidget().set_stage(0)
        elif page_id < self.local_folders_page_id:
            self.sideWidget().ui.states_frame.show()
            self.sideWidget().set_stage(1)
        elif page_id < self.CONGRATULATIONS_PAGE:
            self.sideWidget().set_stage(3)
        else:
            self.sideWidget().set_stage(4)

    # Invalid name "resizeEvent"
    # pylint: disable=C0103

    def resizeEvent(self, event):
        """Resize the overlay to fit all the widget."""
        QtGui.QWizard.resizeEvent(self, event)
        self.overlay.resize(event.size())
