# -*- coding: utf-8 -*-

# Authors: Roberto Alsina <roberto.alsina@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Widget to create UDFs in the Windows Install Wizard."""

import ctypes
import os
import sys
import threading
import Queue

from PyQt4 import QtCore, QtGui

from ubuntuone.controlpanel.gui import humanize

from ubuntu_sso.qt.gui import SSOWizardPage

from ubuntuone_installer.gui.qt.ui import local_folders_ui
from ubuntuone_installer.gui import (
    LOCAL_FOLDERS_TITLE,
    LOCAL_FOLDERS_SPACE_HEADER,
    LOCAL_FOLDERS_OFFER_LABEL,
    LOCAL_FOLDERS_CALCULATING,
)


class FolderItem(QtGui.QTreeWidgetItem):
    """Class representing a folder in the folder list UI."""
    def __init__(self, strings, path=None, queue=None):
        super(FolderItem, self).__init__(strings)
        self.thread = CalculateSize(path, queue)
        self.thread.start()
        self.size = None
        self.path = path


class CalculateSize(threading.Thread):
    """Class to calculate the size of a folder in the background."""
    def __init__(self, path_name, queue):
        self.path_name = path_name
        self.queue = queue
        self._stop = False
        super(CalculateSize, self).__init__()
        self.daemon = True

    def run(self):
        total_size = 0
        for dirpath, _, filenames in os.walk(self.path_name):
            for f in filenames:
                fp = os.path.join(dirpath, f)
                total_size += os.path.getsize(fp)
        self.queue.put([self.path_name, total_size])


class LocalFoldersPage(SSOWizardPage):
    """Wizard page to create UDFs in the Windows Installer."""

    def __init__(self, parent=None):
        super(LocalFoldersPage, self).__init__(
            local_folders_ui.Ui_Form(), None, parent)
        self.setTitle(LOCAL_FOLDERS_TITLE)
        self.setSubTitle()

        header_view = self.ui.folder_list.header()
        header_view.setResizeMode(0, header_view.Stretch)

        self.queue = Queue.Queue()
        self.timer = QtCore.QTimer()
        self.items = {}
        for folder_name in self.default_folders():
            self.add_folder(folder_name)
        self.update_sizes()
        self.timer.start(2000)
        self.timer.timeout.connect(self.update_sizes)

    # initializePage is inherited
    # pylint: disable=C0103
    def initializePage(self):
        """UI details."""
        self.wizard()._next_id = None

    def quota(self):
        """The quota available to the user."""
        # FIXME: get this from real life
        return 2 * 1024 * 1024

    def default_folders(self):
        """Return a list of the folders to add by default."""
        if sys.platform == 'win32':
            # Special Folder "My Documents"
            dll = ctypes.windll.shell32
            buf = ctypes.create_string_buffer(300)
            dll.SHGetSpecialFolderPathA(None, buf, 5, False)
            docs = buf.value
            dll.SHGetSpecialFolderPathA(None, buf, 13, False)
            music = buf.value
            dll.SHGetSpecialFolderPathA(None, buf, 39, False)
            pictures = buf.value
            return [docs, music, pictures]
        else:
            result = ['To be implemented']
        return result

    def add_folder(self, path):
        """Add a folder to the list."""
        if path in self.items:
            return None
        # FIXME: the path should actually be sent to u1cp to verify as valid
        item = FolderItem([path, "", "remove"], path=path, queue=self.queue)
        self.ui.folder_list.addTopLevelItem(item)
        self.items[path] = item
        return item

    def update_sizes(self):
        """Poll the queue were the threads put the size info."""
        try:
            path, size = self.queue.get(False)
            item = self.items.get(path)
            if item:
                item.size = size
                item.setText(1, humanize(size))
        except Queue.Empty:
            pass
        total = 0
        for path, item in self.items.items():
            if item.size is None:
                total = LOCAL_FOLDERS_CALCULATING
                break
            total += item.size

        if isinstance(total, long):
            self.show_hide_offer(total)
            total = humanize(total)
        else:
            self.show_hide_offer(0)
        self.ui.folder_list.headerItem().setText(
            1, LOCAL_FOLDERS_SPACE_HEADER % total)

    def show_hide_offer(self, cur_size):
        """Show or hide the offer to buy space according to the total size."""
        quota = self.quota()

        if cur_size > quota:
            self.ui.offer_frame.setVisible(True)
        else:
            self.ui.offer_frame.setVisible(False)

        self.ui.offer_label.setText(LOCAL_FOLDERS_OFFER_LABEL %
            {"quota": humanize(quota)})

    def stop_threads(self):
        """Stop all pending threads."""
        for _, item in self.items:
            item.thread._stop = True

    # itemClicked is a Qt signal name.
    # pylint: disable=C0103
    def on_folder_list_itemClicked(self, item, column):
        """Delete folder from the list."""
        if column == 2:
            del(self.items[item.path])
            item.thread._stop = True
            self.ui.folder_list.takeTopLevelItem(
                self.ui.folder_list.indexOfTopLevelItem(item))
        self.update_sizes()
