# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#          Roberto Alsina <roberto.alsina@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Qt UI."""

import os
import Queue
import shutil
import tempfile

from twisted.internet import defer

from PyQt4 import QtCore

from ubuntuone.platform.credentials import APP_NAME

from ubuntuone_installer.gui.qt import gui
from ubuntuone_installer.gui.qt.tests import BaseTestCase
from ubuntuone_installer.gui.qt import forgotten
from ubuntuone_installer.gui.qt import local_folders
from ubuntuone_installer.gui.qt import setup_account
from ubuntuone_installer.gui.qt.ui import (
    setup_account_ui,
)


TOKEN = {u'consumer_key': u'xQ7xDAz',
         u'consumer_secret': u'KzCJWCTNbbntwfyCKKjomJDzlgqxLy',
         u'token_name': u'test',
         u'token': u'GkInOfSMGwTXAUoVQwLUoPxElEEUdhsLVNTPhxHJDUIeHCPNEo',
         u'token_secret': u'qFYImEtlczPbsCnYyuwLoPDlPEnvNcIktZphPQklAWrvyfFMV'}


class FakeFailureCredentialsManagementTool(object):

    """A fake CredentialsManagementTool that fails."""

    def find_credentials(self):
        """Return a deferred that never triggers callback."""
        return defer.succeed({})


class FakeController(object):

    """A fake controller for the tests."""

    def __init__(self, *args, **kwargs):
        self.args = (args, kwargs)

    # pylint: disable=C0103
    def setupUi(self, view):
        """Fake the setup."""
    # pylint: enable=C0103
    is_fake = True


class FakeAreYouSure(object):

    """A fake Dialog."""

    result = 0
    shown = False

    def __init__(self, *args, **kwargs):
        self.args = (args, kwargs)

    def exec_(self):
        """Fake showing the dialog."""
        self.shown = True
        return self.result  # Means reject


class FakeOverlay(object):

    """A fake delay overlay."""

    def __init__(self, *args, **kwargs):
        """Initialize."""
        self.show_counter = 0
        self.hide_counter = 0
        self.args = (args, kwargs)

    def show(self):
        """Fake show."""
        self.show_counter += 1

    def hide(self):
        """Fake hide."""
        self.hide_counter += 1

    def resize(self, *args):
        """Fake resize."""


class MainWindowTestCase(BaseTestCase):
    """Test the qt main window."""

    class_ui = gui.MainWindow

    def setUp(self):
        """Initialize this test instance."""
        # Faking each SSO object instead of doing it lower
        # so we don't rely on any SSO behaviour
        self.patch(gui, "SetUpAccountController", FakeController)
        self.patch(gui, "TosController", FakeController)
        self.patch(gui, "EmailVerificationController", FakeController)
        self.patch(gui, "CurrentUserController", FakeController)
        self.patch(gui, "SuccessController", FakeController)
        self.patch(gui, "ForgottenPasswordController", FakeController)
        self.patch(gui, "ResetPasswordController", FakeController)
        self.patch(gui, "AreYouSure", FakeAreYouSure)
        self.patch(gui, "LoadingOverlay", FakeOverlay)
        self.patch(gui, "CredentialsManagementTool",
            FakeFailureCredentialsManagementTool)
        super(MainWindowTestCase, self).setUp()

    def test_critical(self):
        """Show a critical popup, but first hide the overlay."""
        self.assertEqual(self.ui.overlay.hide_counter, 1)
        self.patch(gui.QtGui.QMessageBox, "critical", self._set_called)
        self.ui.critical("WORRY!")
        self.assertEqual(self.ui.overlay.hide_counter, 2)
        self.assertEqual(self._called, (('WORRY!',), {}))

    def test_forgotten_password_controller_error(self):
        """When there is an error, it should hide the overlay."""
        self.assertEqual(self.ui.overlay.hide_counter, 1)
        controller = forgotten.ForgottenPasswordController()
        controller.view = self.ui.forgotten
        controller.on_password_reset_error("app_name",
            {'errtype': 'ResetPasswordTokenError'})
        self.assertEqual(self.ui.overlay.hide_counter, 2)

    def test_overlay_connection_setup_account(self):
        """Make sure we don't lose connections to the overlay."""
        self.assertEqual(self.ui.overlay.show_counter, 0)
        # This test is different from the ones below.
        # For some reason, click() doesn't trigger the connected
        # slot, but clicked.emit() does. The reason is unknown.
        self.ui.setup_account.ui.set_up_button.clicked.emit(False)
        self.assertEqual(self.ui.overlay.show_counter, 1)

    def test_overlay_connection_email_verification(self):
        """Make sure we don't lose connections to the overlay."""
        self.assertEqual(self.ui.overlay.show_counter, 0)
        self.ui.email_verification.next_button.click()
        self.assertEqual(self.ui.overlay.show_counter, 1)

    def test_overlay_connection_current_user(self):
        """Make sure we don't lose connections to the overlay."""
        self.assertEqual(self.ui.overlay.show_counter, 0)
        self.ui.current_user.ui.sign_in_button.click()
        self.assertEqual(self.ui.overlay.show_counter, 1)

    def test_overlay_connection_forgotten_password(self):
        """Make sure we don't lose connections to the overlay."""
        self.assertEqual(self.ui.overlay.show_counter, 0)
        self.ui.forgotten.ui.send_button.click()
        self.assertEqual(self.ui.overlay.show_counter, 1)

    def test_overlay_connection_reset_password(self):
        """Make sure we don't lose connections to the overlay."""
        self.assertEqual(self.ui.overlay.show_counter, 0)
        self.ui.reset_password.ui.reset_password_button.click()
        self.assertEqual(self.ui.overlay.show_counter, 1)

    def test_next_hides_overlay(self):
        """Make sure next() hides the overlay."""
        self.ui.next()
        # This is 2 because we hide it once on __init__()
        # and then again on next()
        self.assertEqual(self.ui.overlay.hide_counter, 2)

    def test_done_calls_custom_close_callback(self):
        """When closing the window, close_callback is called."""
        gui.AreYouSure.result = 0
        self.ui.close_callback = self._set_called
        self.ui.done(result=0)
        self.assertEqual(self._called,
                         ((), {}), 'close_callback called.')

    def test_close_callback_can_be_none(self):
        """The close_callback can be None."""
        gui.AreYouSure.result = 0
        self.ui.close_callback = None
        self.ui.done(result=0)
        # world did not explode

    def test_main_window_general_config(self):
        """Check if the main properties of the windows are set."""
        self.assertEqual(self.ui.windowTitle(), APP_NAME)

    def test_start_control_panel_on_finishing(self):
        """Calling done with result=1, the control panel should be called."""
        gui.AreYouSure.result = 0
        self.patch(gui.subprocess, "Popen", self._set_called)
        self.ui.done(result=1)
        self.assertEqual(self._called,
            ((['ubuntuone-control-panel-qt'],), {}))

    def test_not_start_control_panel_on_cancel(self):
        """Called done with result=0, control panel should NOT be called."""
        gui.AreYouSure.result = 0
        self.patch(gui.subprocess, "Popen", self._set_called)
        self.ui.done(result=0)
        self.assertEqual(self._called, False)

    def test_continue_cancels_cancel(self):
        """Clicking 'Continue' cancels cancellation."""
        gui.AreYouSure.result = 1
        self.patch(gui.subprocess, "Popen", self._set_called)
        self.ui.show()
        self.ui.done(result=0)
        self.assertEqual(self.ui.isVisible(), True)

    def test_user_is_sure_in_last_page(self):
        """Cancelling on the last page is ok."""
        self.ui.show()
        self.patch(self.ui, 'currentId', lambda: self.ui.CONGRATULATIONS_PAGE)
        self.patch(gui.subprocess, "Popen", self._set_called)
        self.ui.done(result=0)
        self.assertEqual(self._called, False)
        self.assertEqual(FakeAreYouSure.shown, False)

    def test_later_accepts_cancel(self):
        """Clicking 'Later' accepts cancellation."""
        gui.AreYouSure.result = 0
        self.ui.show()
        self.ui.done(result=0)
        self.assertEqual(self.ui.isVisible(), False)

    def test_sync_now_shows_message(self):
        """If the user selects 'Sync Now' the congrats page shows a message."""
        congrats_page = self.ui.page(self.ui.CONGRATULATIONS_PAGE)
        congrats_page.setField("sync_now", True)

        # Show the congrats page
        self.ui.setStartId(self.ui.CONGRATULATIONS_PAGE)
        self.ui.restart()
        self.ui.show()

        # Check the properties have the right values
        self.assertEqual(congrats_page.field("sync_later").toBool(),
            False)
        self.assertEqual(
            congrats_page.ui.progressContainer.isVisible(), True)

    def test_sync_later_hides_message(self):
        """Selecting 'Sync Later' the congrats page hides a message."""
        congrats_page = self.ui.page(self.ui.CONGRATULATIONS_PAGE)
        congrats_page.setField("sync_later", True)

        # Show the congrats page
        self.ui.setStartId(self.ui.CONGRATULATIONS_PAGE)
        self.ui.restart()
        self.ui.show()

        # Check the properties have the right values
        self.assertEqual(congrats_page.field("sync_now").toBool(),
            False)
        self.assertEqual(
            congrats_page.ui.progressContainer.isVisible(), False)

    def test_current_user_controller_parameters(self):
        """Compare controller parameters with expected values."""
        self.assertEqual(self.ui.current_user_controller.args,
        ((), {'message_box': self.ui,
              'title': gui.TITLE_STYLE % gui.SIGN_IN,
              'subtitle': gui.SIGN_IN_SUBTITLE}))

    def test_stage_progression_1(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.LICENSE_PAGE_ID)
        self.assertEqual(self.ui.sideWidget().stage, 0)

    def test_stage_progression_2(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.SIGNIN_PAGE)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_3(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.setup_account_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_4(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.tos_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_5(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.email_verification_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_6(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.current_user_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_7(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.SUCCESS_PAGE)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_8(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.ERROR_PAGE)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_9(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.forgotten_password_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_10(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.reset_password_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_11(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.local_folders_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 3)

    def test_stage_progression_12(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.SYNC_NOW_OR_LATER_PAGE)
        self.assertEqual(self.ui.sideWidget().stage, 3)

    def test_stage_progression_13(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.CONGRATULATIONS_PAGE)
        self.assertEqual(self.ui.sideWidget().stage, 4)

    def test_choose_signin_cancel(self):
        """Test that the custom cancel button works."""
        self.ui.sign_in_page.ui.cancel_button.click()
        QtCore.QCoreApplication.instance().processEvents()
        self.assertEqual(self.ui.result(), self.ui.Rejected)

    def test_license_next(self):
        """Test what's the license page's nextID."""
        self.ui.creds_cb("fakecreds")
        self.assertEqual(self.ui.LICENSE_PAGE.nextId(),
            self.ui.local_folders_page_id)


class FakeSignal(object):

    """A fake PyQt signal."""

    def __init__(self, *args, **kwargs):
        """Initialize."""
        self.target = lambda *args: None

    def connect(self, target):
        """Fake connect."""
        self.target = target

    def emit(self, *args):
        """Fake emit."""
        self.target(*args)


class FakeMainWindow(object):

    """A fake MainWindow."""

    loginSuccess = FakeSignal()
    registrationSuccess = FakeSignal()
    userCancellation = FakeSignal()
    shown = False

    def show(self):
        """Fake method."""
        self.shown = True


class LocalFoldersTestCase(BaseTestCase):
    """Test the LocalFoldersPage code."""

    class_ui = local_folders.LocalFoldersPage

    def setUp(self):
        """Initialize this test instance."""
        # Create a test folder with a known size
        self.tmpdir = tempfile.mkdtemp()
        f = open(os.path.join(self.tmpdir, 'test_file'), 'wb')
        f.write(" " * 600)
        f.close()
        os.mkdir(os.path.join(self.tmpdir, 'test_dir'))
        f = open(os.path.join(self.tmpdir, 'test_dir', 'test_file_2'), 'wb')
        f.write(" " * 737)
        f.close()
        super(LocalFoldersTestCase, self).setUp()

    def tearDown(self):
        """Remove the temporary tree."""
        shutil.rmtree(self.tmpdir)
        BaseTestCase.tearDown(self)

    def test_size_calculation(self):
        """Test the recursive folder size calculation."""
        queue = Queue.Queue()
        csize = local_folders.CalculateSize(self.tmpdir, queue)
        csize.run()
        path, size = queue.get()
        self.assertEqual(path, self.tmpdir)
        self.assertEqual(size, 1337)

    def test_item_addition(self):
        """Add an item (plus the default one), then remove them."""
        self.ui.add_folder(self.tmpdir)
        self.assertEqual(4, self.ui.ui.folder_list.topLevelItemCount())

    def test_total_size(self):
        """Test that the header reflects the change in item sizes."""
        while self.ui.ui.folder_list.topLevelItemCount():
            self.ui.on_folder_list_itemClicked(
                self.ui.ui.folder_list.topLevelItem(0), 2)
        self.assertEqual(0, self.ui.ui.folder_list.topLevelItemCount())
        item = self.ui.add_folder(self.tmpdir)
        item.size = 1337
        item.thread.run()
        item.thread.join()
        self.ui.update_sizes()
        self.assertEqual(unicode(self.ui.ui.folder_list.headerItem().text(1)),
            u"Space (1337)")

    def test_add_twice(self):
        """Behaviour for adding the same folder twice:

        * It's added only once.
        """
        while self.ui.ui.folder_list.topLevelItemCount():
            self.ui.on_folder_list_itemClicked(
                self.ui.ui.folder_list.topLevelItem(0), 2)
        self.assertEqual(0, self.ui.ui.folder_list.topLevelItemCount())
        self.ui.add_folder(self.tmpdir)
        self.ui.add_folder(self.tmpdir)
        self.assertEqual(1, self.ui.ui.folder_list.topLevelItemCount())

    def test_add_missing_folder(self):
        """Behaviour for adding a folder that doesn't exist:

        * It's added.
        * Has size 0.
        """

        while self.ui.ui.folder_list.topLevelItemCount():
            self.ui.on_folder_list_itemClicked(
                self.ui.ui.folder_list.topLevelItem(0), 2)
        self.assertEqual(0, self.ui.ui.folder_list.topLevelItemCount())
        item = self.ui.add_folder(os.path.join("xyzzy", "xyzzy", "xyzzy"))
        self.assertEqual(1, self.ui.ui.folder_list.topLevelItemCount())
        item.thread.run()
        item.thread.join()
        self.ui.update_sizes()
        self.assertEqual(0, item.size)
        # world did not explode

    def test_over_quota(self):
        """After removing all folders, offer_frame should be hidden.

        Push the user over quota, it should be visible.
        """
        self.patch(self.ui.ui.offer_frame, "setVisible", self._set_called)
        while self.ui.ui.folder_list.topLevelItemCount():
            self.ui.on_folder_list_itemClicked(
                self.ui.ui.folder_list.topLevelItem(0), 2)
        self.assertEqual(0, self.ui.ui.folder_list.topLevelItemCount())
        self.ui.update_sizes()
        self.assertEqual(self._called, ((False,), {}))
        self.ui.show_hide_offer(self.ui.quota() + 1)
        self.assertEqual(self._called, ((True,), {}))


class SetupAccountTestCase(BaseTestCase):
    """Test the SetupAccountPage code."""

    class_ui = setup_account.SetupAccountPage
    kwargs = dict(
        ui=setup_account_ui.Ui_SetUpAccountPage(),
        controller=FakeController(),
        parent=None,
    )

    def setUp(self):
        """Initialize this test instance."""
        super(SetupAccountTestCase, self).setUp()

    def test_empty_name(self):
        """Status when the name field is empty:

        * Name Assistance label visible
        * Correct error message in it
        """
        self.ui.ui.name_edit.setText("")
        self.ui.name_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.name_assistance.isVisible(), True)
        self.assertEqual(
            unicode(self.ui.ui.name_assistance.text()),
            setup_account.BAD % setup_account.EMPTY_NAME)
        self.ui.hide()

    def test_blank_name(self):
        """Status when the name field is blank (spaces):

        * Name Assistance label visible
        * Correct error message in it
        """
        self.ui.ui.name_edit.setText("  ")
        self.ui.name_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.name_assistance.isVisible(), True)
        self.assertEqual(
            unicode(self.ui.ui.name_assistance.text()),
            setup_account.BAD % setup_account.EMPTY_NAME)
        self.ui.hide()

    def test_valid_name(self):
        """Status when the name field is valid:

        * Name Assistance label invisible
        """
        self.ui.ui.name_edit.setText("John Doe")
        self.ui.name_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.name_assistance.isVisible(), False)
        self.ui.hide()

    def test_invalid_email(self):
        """Status when the email field has no @:

        * Email Assistance label visible
        * Correct error message in it
        """
        self.ui.ui.email_edit.setText("foobar")
        self.ui.email_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.email_assistance.isVisible(), True)
        self.assertEqual(
            unicode(self.ui.ui.email_assistance.text()),
            setup_account.BAD % setup_account.INVALID_EMAIL)
        self.ui.hide()

    def test_valid_email(self):
        """Status when the email field has a @:

        * Email Assistance label invisible
        """
        self.ui.ui.email_edit.setText("foo@bar")
        self.ui.email_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.email_assistance.isVisible(), False)
        self.ui.hide()

    def test_matching_emails(self):
        """Status when the email fields match:

        * Email Assistance label invisible
        """
        self.ui.ui.email_edit.setText("foo@bar")
        self.ui.ui.confirm_email_edit.setText("foo@bar")
        self.ui.confirm_email_assistance()
        self.ui.show()
        self.assertEqual(
            self.ui.ui.confirm_email_assistance.isVisible(), False)
        self.ui.hide()

    def test_not_matching_emails(self):
        """Status when the email fields don't match:

        * Email Assistance label visible
        * Correct error message
        """
        self.ui.ui.email_edit.setText("foo@bar")
        self.ui.ui.confirm_email_edit.setText("foo@baz")
        self.ui.confirm_email_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.confirm_email_assistance.isVisible(), True)
        self.assertEqual(
            unicode(self.ui.ui.confirm_email_assistance.text()),
            setup_account.BAD % setup_account.EMAIL_MATCH)
        self.ui.hide()

    def test_short_password(self):
        """Status with short password:

        * Password assistance contains the right message
        """
        self.ui.ui.password_edit.setText("foobar")
        self.ui.password_assistance()
        self.assertEqual(
            True,
            setup_account.BAD % setup_account.PASSWORD_LENGTH in
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_long_password(self):
        """Status with long password:

        * Password assistance contains the right message
        """
        self.ui.ui.password_edit.setText("foobarbaz")
        self.ui.password_assistance()
        self.assertEqual(
            True,
            setup_account.GOOD % setup_account.PASSWORD_LENGTH in
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_no_number_password(self):
        """Status with password without a number:

        * Password assistance contains the right message
        """
        self.ui.ui.password_edit.setText("foobarbaz")
        self.ui.password_assistance()
        self.assertEqual(
            True,
            setup_account.BAD % setup_account.PASSWORD_DIGIT in
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_number_password(self):
        """Status with password with a number:

        * Password assistance contains the right message
        """
        self.ui.ui.password_edit.setText("foobarba7")
        self.ui.password_assistance()
        self.assertEqual(
            True,
            setup_account.GOOD % setup_account.PASSWORD_DIGIT in
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_no_uppercase_password(self):
        """Status with password without uppercase letters:

        * Password assistance contains the right message
        """
        self.ui.ui.password_edit.setText("foobarbaz")
        self.ui.password_assistance()
        self.assertEqual(
            True,
            setup_account.BAD % setup_account.PASSWORD_UPPER in
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_upper_password(self):
        """Status with password with uppercase letters:

        * Password assistance contains the right message
        """
        self.ui.ui.password_edit.setText("Foobarba7")
        self.ui.password_assistance()
        self.assertEqual(
            True,
            setup_account.GOOD % setup_account.PASSWORD_UPPER in
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_matching_passwords(self):
        """Status when the passwords match:

        * Password assistance doesn't contain the message
        """
        self.ui.ui.password_edit.setText("Foobarba7")
        self.ui.ui.confirm_password_edit.setText("Foobarba7")
        self.ui.password_assistance()
        self.assertEqual(
            False,
            setup_account.PASSWORD_MATCH in
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_not_matching_passwords(self):
        """Status when the passwords not match:

        * Password assistance contains the right message
        """
        self.ui.ui.password_edit.setText("Foobarba7")
        self.ui.ui.confirm_password_edit.setText("BazBarFo0")
        self.ui.password_assistance()
        self.assertEqual(
            True,
            setup_account.BAD % setup_account.PASSWORD_MATCH in
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_side_widget_state(self):
        """Test if the side widget of the wizard has state frame visible."""
        if type(self.ui) != gui.MainWindow:
            return
        self.ui.on_currentIdChanged(self.ui.LICENSE_PAGE)
        self.assertEqual(
            True,
            self.ui.sideWidget().ui.states_frame.isVisible())
        self.ui.on_currentIdChanged(self.ui.SIGNIN_PAGE)
        self.assertEqual(
            True,
            self.ui.sideWidget().ui.states_frame.isVisible())
        self.ui.on_currentIdChanged(self.ui.local_folders_page_id)
        self.assertEqual(
            True,
            self.ui.sideWidget().ui.states_frame.isVisible())
        self.ui.on_currentIdChanged(self.ui.CONGRATULATIONS_PAGE)
        self.assertEqual(
            True,
            self.ui.sideWidget().ui.states_frame.isVisible())
